@@ -10,7 +10,7 @@ use Rose::DB::Object::Metadata::UniqueKey;
 use Rose::DB::Object::Metadata::Auto;
 our @ISA = qw(Rose::DB::Object::Metadata::Auto);
 
-our $VERSION = '0.021';
+our $VERSION = '0.023';
 
 # syscolumns.coltype constants taken from:
 #
@@ -143,7 +143,8 @@ sub auto_generate_columns
     #   4     data type (encoded)
     #   5     data length (encoded)
     #
-    my @col_list = DBD::Informix::Metadata::ix_columns($dbh, $self->table);
+    # Lowercase table because Rose::DB::Informix->likes_lowercase_table_names
+    my @col_list = DBD::Informix::Metadata::ix_columns($dbh, lc $self->table);
 
     # We'll also need to query the syscolumns table directly to get the
     # table id, which we need to query the sysdefaults table.  But to get
@@ -467,7 +468,7 @@ sub auto_retrieve_primary_key_column_names
     #   4     data type (encoded)
     #   5     data length (encoded)
     #
-    my @col_list = DBD::Informix::Metadata::ix_columns($dbh, $self->table);
+    my @col_list = DBD::Informix::Metadata::ix_columns($dbh, lc $self->table);
 
     my $owner = $col_list[0][0];
     my $table = $col_list[0][1]; # just in case...
@@ -574,7 +575,7 @@ sub auto_generate_unique_keys
     #   4     data type (encoded)
     #   5     data length (encoded)
     #
-    my @col_list = DBD::Informix::Metadata::ix_columns($dbh, $self->table);
+    my @col_list = DBD::Informix::Metadata::ix_columns($dbh, lc $self->table);
 
     # Here's the query for the table id
     my $st_sth = $dbh->prepare(<<"EOF");
@@ -793,7 +794,7 @@ sub auto_generate_foreign_keys
     my $dbh = $db->dbh or die $db->error;
 
     # I'm doing this to get the table id and owner.  Gotta be a better way...
-    my @col_list = DBD::Informix::Metadata::ix_columns($dbh, $self->table);
+    my @col_list = DBD::Informix::Metadata::ix_columns($dbh, lc $self->table);
 
     my $st_sth = $dbh->prepare(<<"EOF");
 SELECT tabid FROM informix.systables WHERE tabname = ? AND owner = ?
@@ -862,6 +863,27 @@ EOF
 
       unless($foreign_class)
       {
+        # Add deferred task
+        $self->add_deferred_task(
+        {
+          class  => $self->class, 
+          method => 'auto_init_foreign_keys',
+          args   => \%args,
+
+          code   => sub
+          {
+            $self->auto_init_foreign_keys(%args);
+            $self->make_foreign_key_methods(%args, preserve_existing => 1);
+          },
+
+          check  => sub
+          {
+            my $num = scalar @foreign_keys;
+            my $fks = $self->foreign_keys;
+            return @$fks > $num ? 1 : 0;
+          }
+        });
+
         unless($no_warnings)
         {
           no warnings; # Allow undef coercion to empty string
